#pragma once

#include <math.h>
#include <iostream>
#include <cassert>

#define STB_IMAGE_WRITE_IMPLEMENTATION
#include <stb_image_write.hpp>

#include "util.hpp"
#include "Renderer.hpp"

namespace massivedynamic {

	class PixelRenderer : public Renderer {
		protected:
		 	size_t pixelSize;
			size_t border;
			std::vector<Color> bitmap;

			void drawPixelScaled(size_t x, size_t y, Color color) {
				size_t pixelSize = this->pixelSize;

				assert(pixelSize > 1 && "ERROR: output file size is too small");

				size_t absoluteX = pixelSize * x + this->border;
				size_t absoluteY = pixelSize * y + this->border;

				for (size_t localY = absoluteY; localY < absoluteY + pixelSize; localY++) {
					for (size_t localX = absoluteX; localX < absoluteX + pixelSize; localX++) {
						this->bitmap.at(localY * this->targetSize + localX) = color;
					}
				}
			}
		public:
			enum Colors : Color {
				BLACK = 0xFF000000,
				WHITE = 0xFFFFFFFF
			};

			PixelRenderer(const std::vector<bool>& pixels, size_t sourceSize, size_t targetSize) :
					Renderer(pixels, sourceSize, targetSize == 0 ? (sourceSize + 2) * 2 : targetSize),
					pixelSize(round(static_cast<float>(this->targetSize) / static_cast<float>(this->sourceSize + 2))),
					border(round(static_cast<float>(this->targetSize - this->pixelSize * this->sourceSize) / 2.0f)),
					bitmap(std::vector<Color>(this->targetSize * this->targetSize, Colors::WHITE)) {}

			virtual void generateBuffer() {
				for (size_t y = 0; y < sourceSize; y++) {
					for (size_t x = 0; x < sourceSize; x++) {
						if (!pixels.at(y * sourceSize + x))
							continue;
						this->drawPixelScaled(x, y, Colors::BLACK);
					}
				}

			}
	};

}
